"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Timezone = void 0;
var tslib_1 = require("tslib");
var moment_timezone_1 = (0, tslib_1.__importDefault)(require("moment-timezone"));
var Timezone = (function () {
    function Timezone(timezone) {
        if (typeof timezone !== 'string') {
            throw new TypeError('timezone description must be a string');
        }
        if (timezone !== 'Etc/UTC' && !moment_timezone_1.default.tz.zone(timezone)) {
            throw new Error("timezone '".concat(timezone, "' does not exist"));
        }
        this.timezone = timezone;
    }
    Timezone.formatDateWithTimezone = function (d, timezone) {
        var str;
        if (timezone && !timezone.isUTC()) {
            str = moment_timezone_1.default.tz(d, timezone.toString()).format('YYYY-MM-DDTHH:mm:ss.SSSZ');
        }
        else {
            str = d.toISOString();
        }
        return str.replace('.000', '');
    };
    Timezone.fromJS = function (spec) {
        return new Timezone(spec);
    };
    Timezone.prototype.valueOf = function () {
        return this.timezone;
    };
    Timezone.prototype.toJS = function () {
        return this.timezone;
    };
    Timezone.prototype.toJSON = function () {
        return this.timezone;
    };
    Timezone.prototype.toString = function () {
        return this.timezone;
    };
    Timezone.prototype.equals = function (other) {
        return other instanceof Timezone && this.timezone === other.timezone;
    };
    Timezone.prototype.isUTC = function () {
        return this.timezone === 'Etc/UTC';
    };
    Timezone.prototype.toUtcOffsetString = function () {
        var utcOffset = moment_timezone_1.default.tz(this.toString()).utcOffset();
        var hours = String(Math.abs(Math.floor(utcOffset / 60))).padStart(2, '0');
        var minutes = String(Math.abs(utcOffset % 60)).padStart(2, '0');
        return "UTC ".concat(utcOffset >= 0 ? '+' : '-').concat(hours, ":").concat(minutes);
    };
    Timezone.UTC = new Timezone('Etc/UTC');
    return Timezone;
}());
exports.Timezone = Timezone;
